"""Test the tools module."""


from proselint.config import default
from proselint.tools import lint as proselint, load_options

from .check import Check


def lint(text):
    return proselint(text, config=load_options(conf_default=default))


class TestLint(Check):
    """The test class for tools.lint."""

    __test__ = True

    def setUp(self):
        """setUp method creating text fixtures."""
        self.text = """But this is a very bad sentence.
This is also a no-good sentence.

"""
        self.text_with_no_newline = """A very bad sentence."""

    def extract_line_col(self, error):
        """Extract the line and column number from an error tuple."""
        _, _, line, column, _, _, _, _, _ = error
        return line, column

    def test_errors_sorted(self):
        """Test that errors are sorted by line and column number."""
        lines_and_cols = [self.extract_line_col(e) for e in lint(self.text)]
        assert sorted(lines_and_cols) == lines_and_cols

    def test_on_no_newlines(self):
        """Test that lint works on text without a terminal newline."""
        assert len(lint(self.text_with_no_newline)) == 1



"""Test user option overrides using --config and load_options"""
import json
import os
from pathlib import Path
from unittest import TestCase
from unittest.mock import patch

from click.testing import CliRunner

from proselint.command_line import proselint
from proselint.config import default
from proselint.tools import deepmerge_dicts, load_options

runner = CliRunner()

CONFIG_FILE = str(Path(__file__, "../test-proselintrc.json").resolve())
FLAG = f"--config '{CONFIG_FILE}'"


def test_deepmerge_dicts():
    """Test deepmerge_dicts"""
    d1 = {'a': 1, 'b': {'c': 2, 'd': 3}}
    d2 = {'a': 2, 'b': {'c': 3, 'e': 4}}
    assert deepmerge_dicts(d1, d2) == {'a': 2, 'b': {'c': 3, 'd': 3, 'e': 4}}


@patch("os.path.isfile")
def test_load_options_function(isfile):
    """Test load_options by specifying a user options path"""

    isfile.side_effect = CONFIG_FILE.__eq__

    overrides = load_options(CONFIG_FILE, default)
    assert load_options(conf_default=default)["checks"]["uncomparables.misc"]
    assert not overrides["checks"]["uncomparables.misc"]

    isfile.side_effect = os.path.join(os.getcwd(), ".proselintrc.json").__eq__

    TestCase().assertRaises(FileNotFoundError, load_options)


def test_config_flag():
    """Test the --config CLI argument"""
    output = runner.invoke(proselint, "--demo")
    assert "uncomparables.misc" in output.stdout

    output = runner.invoke(proselint, f"--demo {FLAG}")
    assert "uncomparables.misc" not in output.stdout
    assert "FileNotFoundError" != output.exc_info[0].__name__

    output = runner.invoke(proselint, "--demo --config non_existent_file")
    assert output.exit_code == 1
    assert "FileNotFoundError" == output.exc_info[0].__name__

    output = runner.invoke(proselint, "non_existent_file")
    assert output.exit_code == 2


def test_dump_config():
    """Test --dump-default-config and --dump-config"""
    output = runner.invoke(proselint, "--dump-default-config")
    assert json.loads(output.stdout) == default

    output = runner.invoke(proselint, f"--dump-config {FLAG}")
    assert json.loads(output.stdout) == json.load(open(CONFIG_FILE))



#!/usr/bin/env python

"""Test GMEU entry 'a', part A."""


from proselint.tools import assert_error

from tests.check import Check


class chk:
    def check(self, text):
        return assert_error, text, "misc.a_vs_an"


class TestCheck(Check):
    """The test class for GMEU entry A - using the correct determiner."""

    @property
    def this_check(self):
        return chk

    sentences_with_errors = [
        "Are you an Yankee fan?",
        "Coffee tastes less sweet in an white mug.",
        "One of them wore a opalescent vest.",
        "You're a intelligent guy, often misguided.",
        "Ezra gave an eulogy.",
        "What capital of an European country is the farthest north?",
        "His sole reward was an one-year term as Ambassador to Thailand.",
        "I will be relying on a roulette wheel and an Ouija board.",
        "Anyone in an uniform is fair game.",
        "Grimm started working as a F.B.I. agent in 1995.",
        "Out of 186 managers participating, 57 had a MBA degree.",
        "Smith announced that a SEC filing is pending.",
        "This argument is an historical desecration.",
        "The treatment of crime in Britain shows an historic shift away…",
        "It is, in some ways, an humble form.",
        "The thief turned out to be an habitual offender from Darlington.",
        "This stage displays an hallucinatory image that signifies itself.",
        "He saw an hallucinatory image before passing out.",
        "A triumphant Adolf Hitler addressed an hysterical crowd.",
        "Kun pieced together an history of gender-segregated dining in L.A.",
        "An historian who fled the Nazis and still wants us to read Hitler.",
        "It is nominally an historical novel.",
        "It feels good to validate an hypothesis.",
        "An hereditary title can be passed to a member of the family.",
        "The Clinton Presidency was an historic era of prosperity.",
        "There comes to be an habitual pattern between neurons in the brain.",
        "They had the authority to start an humanitarian intervention.",
        "She laughed aloud, an hysterical sort of giggled, quickly stifled",
    ]

    def test_smoke(self):
        for sentence in self.sentences_with_errors:
            assert not self.passes(sentence)



"""Tests for dates_times.am_pm check."""

from proselint.checks.dates_times import am_pm as chk

from .check import Check


class TestCheck(Check):
    """The test class for dates_times.am_pm."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke_check_lowercase_periods(self):
        """Basic smoke test.

        This is for the function
        dates_times.am_pm.check_lowercase_periods.

        """
        assert chk.check_lowercase_periods(
            "Basic smoke phrase without issues.") == []
        assert chk.check_lowercase_periods(
            "It happened at 7 a.m.") == []
        assert chk.check_lowercase_periods(
            "It happened at 7 am.") != []
        assert chk.check_lowercase_periods(
            "On Wed, Sep 21, 2016 at 11:42 AM -0400, X wrote:") == []

    def test_smoke_check_spacing(self):
        """Basic smoke test.

        This is for the function
        dates_times.am_pm.check_spacing.

        """
        assert chk.check_spacing(
            "Basic smoke phrase without issues.") == []
        assert chk.check_spacing(
            "It happened at 7 a.m.") == []
        assert chk.check_spacing(
            "It happened at 7a.m.") != []

    def test_smoke_check_midnight_noon(self):
        """Basic smoke test.

        This for the function
        dates_times.am_pm.midnight_noon.

        """
        assert chk.check_midnight_noon(
            "Basic smoke phrase without issues.") == []
        assert chk.check_midnight_noon(
            "It happened at noon.") == []
        assert chk.check_midnight_noon(
            "It happened at 12 a.m.") != []

    def test_smoke_check_redundancy(self):
        """Basic smoke test.

        This for the function
        dates_times.am_pm.check_redundancy.

        """
        assert len(chk.check_redundancy(
            "Basic smoke phrase without issues.")) == 0
        assert len(chk.check_redundancy(
            "It happened at 7 a.m.")) == 0
        assert len(chk.check_redundancy(
            "It happened at 7a.m. in the morning.")) == 1



"""Password in plain text.

---
layout:     post
source:     ???
source_url: ???
title:      password in plain text
date:       2014-06-10 12:31:19
categories: writing
---

Don't put pass
"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "security.password"
    msg = "Don't put passwords in plain text."

    pwd_regex = r"[:]? [\S]{6,30}"

    password = [
        f"the password is{pwd_regex}",
        f"my password is{pwd_regex}",
        f"the password's{pwd_regex}",
        f"my password's{pwd_regex}",
        f"^[pP]assword{pwd_regex}",
    ]

    return existence_check(text, password, err, msg)



"""Tests for security.credit_card check."""

from proselint.checks.security import credit_card as chk

from .check import Check


class TestCheck(Check):
    """The test class for security.credit_card."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for security.credit_card.

        This makes use of a test MasterCard number.
        """
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes(
            """My credit card number is 5555555555554444.""")



"""Tests for the clear cache operation from proselint.command_line."""

import os
import unittest

from proselint import command_line as cl

try:
    from unittest import mock
except ImportError:
    # Py2.x
    from unittest import mock

try:
    from builtins import PermissionError
except ImportError:

    class PermissionError(OSError):
        """Introduced in Py3.3, emulate for earlier versions."""

        def __init__(self, *args, **kwargs):
            """Constructor."""
            OSError.__init__(self, *args, **kwargs)

try:
    from builtins import FileNotFoundError
except ImportError:

    class FileNotFoundError(OSError):
        """Introduced in Py3.3, emulate for earlier versions."""

        def __init__(self, *args, **kwargs):
            """Constructor."""
            OSError.__init__(self, *args, **kwargs)

try:
    from builtins import IsADirectoryError
except ImportError:

    class IsADirectoryError(OSError):
        """Introduced in Py3.3, emulate for earlier versions."""

        def __init__(self, *args, **kwargs):
            """Constructor."""
            OSError.__init__(self, *args, **kwargs)


class Test__delete_compiled_python_files(unittest.TestCase):
    """proselint.command_line._delete_compiled_python_files()."""

    def setUp(self):
        """init common data."""
        self.base_dir = '.'
        self.python_file = 'a.py'
        self.pyc_file = 'a.pyc'
        self.dot_pyc = '.pyc'

        self.files = [
            (self.base_dir, ('dummy',), (self.pyc_file,
                                         self.python_file,
                                         self.dot_pyc))
        ]

        self.pyc_file_path = os.path.join(self.base_dir, self.pyc_file)
        self.python_file_path = os.path.join(self.base_dir, self.python_file)
        self.dot_pyc_path = os.path.join(self.base_dir, self.python_file)

    @mock.patch('os.walk')
    @mock.patch('os.remove')
    def test_delete_pyc_file(self, mock_remove, mock_walk):
        """Ensure 'pyc' files are removed."""
        mock_walk.return_value = self.files

        cl._delete_compiled_python_files()
        mock_remove.assert_called_with(self.pyc_file_path)

    @mock.patch('os.walk')
    @mock.patch('os.remove')
    def test_files_not_deleted(self, mock_remove, mock_walk):
        """Ensure non 'pyc' files are not removed."""
        mock_walk.return_value = self.files
        cl._delete_compiled_python_files()

        with self.assertRaises(AssertionError):
            mock_remove.assert_called_with(self.python_file_path)

        with self.assertRaises(AssertionError):
            mock_remove.assert_called_with(self.dot_pyc_path)

    @mock.patch('os.walk')
    @mock.patch('os.remove', side_effect=PermissionError)
    def test_no_permission(self, mock_remove, mock_walk):
        """Ignore if unable to delete files."""
        mock_walk.return_value = self.files
        cl._delete_compiled_python_files()

    @mock.patch('os.walk')
    @mock.patch('os.remove', side_effect=OSError)
    def test_on_oserror(self, mock_remove, mock_walk):
        """Ignore if OSError."""
        mock_walk.return_value = self.files
        cl._delete_compiled_python_files()

    @mock.patch('os.walk')
    @mock.patch('os.remove', side_effect=FileNotFoundError)
    def test_files_not_found(self, mock_remove, mock_walk):
        """Ignore if file not found."""
        mock_walk.return_value = self.files
        cl._delete_compiled_python_files()

    @mock.patch('os.walk')
    @mock.patch('os.remove', side_effect=IsADirectoryError)
    def test__remove_dir(self, mock_remove, mock_walk):
        """Ignore if attempt to delete a directory."""
        mock_walk.return_value = self.files
        cl._delete_compiled_python_files()


class Test__delete_cache(unittest.TestCase):
    """proselint.command_line.__delete_cache()."""

    def setUp(self):
        """Init common data."""
        self.cache_path = os.path.join("proselint", "cache")

    @mock.patch('shutil.rmtree')
    def test_rm_cache(self, mock_rmtree):
        """Correct directory is removed."""
        cl._delete_cache()
        mock_rmtree.assert_called_with(self.cache_path)

    @mock.patch('shutil.rmtree', side_effect=PermissionError)
    def test_no_permission(self, mock_rmtree):
        """Ignore if unable to delete."""
        cl._delete_cache()

    @mock.patch('shutil.rmtree', side_effect=OSError)
    def test_on_oserror(self, mock_rmtree):
        """Ignore if general OSError."""
        cl._delete_cache()



"""Test the Cliches.misc module."""

from proselint.checks.cliches import misc as chk

from .check import Check


class TestCheck(Check):
    """The test class for cliches.misc."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def setUp(self):
        """Create test sentences."""
        self.l_garner = """Worse than a fate worse than death."""
        self.l_write_good = """He's a chip off the old block."""
        self.l_gnu_diction = """It's a matter of concern."""

    def test_cliches_garner_basic(self):
        """Basic checks on check_cliches_garner."""
        assert chk.check_cliches_garner("""No cliches here.""") == []
        # use one of the example cliches to verify basic functionality
        assert chk.check_cliches_garner(self.l_garner) != []
        assert "cliches.garner" in chk.check_cliches_garner(self.l_garner)[0]

    def test_cliches_write_good_basic(self):
        """Basic checks on check_cliches_write_good."""
        assert chk.check_cliches_write_good("""No cliches here.""") == []
        # use one of the example cliches to verify basic functionality
        assert chk.check_cliches_write_good(self.l_write_good) != []
        assert "cliches.write_good" in chk.check_cliches_write_good(
            self.l_write_good)[0]

    def test_cliches_gnu_diction_basic(self):
        """Basic check on check_cliches_gnu_diction."""
        assert chk.check_cliches_gnu_diction("""No cliches here.""") == []
        # use one of the example cliches to verify basic functionality
        assert chk.check_cliches_gnu_diction(self.l_gnu_diction) != []
        assert "cliches.gnu_diction" in chk.check_cliches_gnu_diction(
            self.l_gnu_diction)[0]



"""Tests for misc.false_plurals check."""

from proselint.checks.misc import false_plurals as chk

from .check import Check


class TestCheck(Check):
    """The test class for misc.false_plurals."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for misc.false_plurals."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""There were several phenomenons.""")

    def test_smoke_kudos(self):
        """Basic smoke test for misc.false_plurals.kudos."""
        assert chk.check_kudos("""Smoke phrase with nothing flagged.""") == []
        assert chk.check_kudos("""I give you many kudos.""") != []



"""Tests for redundancy.misc check."""

from proselint.checks.redundancy import misc as chk

from .check import Check


class TestCheck(Check):
    """The test class for redundancy.misc."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke_check(self):
        """Basic smoke test for redundancy.misc.check."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""The table was rectangular in shape.""")

    def test_smoke_garner(self):
        """Basic smoke test for redundancy.misc.check_garner."""
        assert chk.check_garner(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_garner(
            """It was blatantly obvious what to do next.""") != []

    def test_smoke_nordquist(self):
        """Basic smoke test for redundancy.misc.check_norquist."""
        assert chk.check_nordquist(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_nordquist(
            """Taking the package was absolutely essential.""") != []

    def test_smoke_atd(self):
        """Basic smoke test for redundancy.misc.check_norquist."""
        assert chk.check_atd(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_atd(
            """He often repeated the old adage.""") != []



"""Don't start a paragraph with 'But'.

---
layout:
source: Justin Jungé
source_url:
title:
date:       2016-03-10 12:31:19
categories: writing
---

Paragraphs should not start with certain bad words.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Do not start a paragraph with a 'But'."""
    err = "misc.but"
    msg = "No paragraph should start with a 'But'."
    regex = r"(^|([\n\r]+))(\s*)But"

    return existence_check(text, [regex], err, msg, ignore_case=False)



"""Test version number."""
from click.testing import CliRunner

from proselint.command_line import proselint
from proselint.version import __version__

from .check import Check


class TestCheck(Check):
    """Test class for version number."""

    __test__ = True

    def test_version(self):
        """Make sure the version number is correct."""
        runner = CliRunner()

        output = runner.invoke(proselint, "--version")
        assert __version__ in output.stdout



"""Test the preferred_forms_check function from the tools.py module."""

from proselint.tools import preferred_forms_check as chk

from .check import Check


class TestCheck(Check):
    """The test class for tools.preferred_forms_check."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def setUp(self):
        """Create some test fixtures."""
        self.L = [['use', ['utilize']]]
        self.l_caps = [["Stone Age",  ["stone age"]]]
        self.err = 'error message'
        self.msg = 'use the preferred form'

    def test_smoke(self):
        """Basic smoke test for preferred_forms_check."""
        assert chk(
            "We utilize this tech", self.L, self.err, self.msg) != []
        assert chk(
            "We use this tech", self.L, self.err, self.msg) == []

    def test_capitalization(self):
        """Test for preferred forms involving capitalization."""
        assert not chk(
            "In the stone age", self.l_caps, self.err, self.msg,
            ignore_case=False)
        assert chk(
            "In the Stone Age", self.l_caps, self.err, self.msg,
            ignore_case=False) == []



"""Tests for misc.capitalization check."""

from proselint.checks.misc import capitalization as chk

from .check import Check


class TestCheck(Check):
    """The test class for misc.capitalization."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for misc.capitalization.check."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""It goes back to the stone age.""")

    def test_smoke_check_months(self):
        """Basic smoke test for misc.capitalization.check_months."""
        assert chk.check_months("""Smoke phrase with nothing flagged""") == []
        assert chk.check_months("""A nice day in june.""") != []

    def test_smoke_check_days(self):
        """Basic smoke test for misc.capitalization.check_days."""
        assert chk.check_days("""Smoke phrase with nothing flagged""") == []
        assert chk.check_days("""It happened on friday.""") != []



"""-ance vs. -ence."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """-ance vs. -ence."""
    err = "spelling.ance_ence"
    msg = "-ance vs. -ence. '{}' is the correct spelling."

    preferences = [

        ["appearance", ["appearence"]],
        ["occurrence", ["occurrance"]],
        ["resistance", ["resistence"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



"""Tests for consistency.spelling check."""

from proselint.checks.consistency import spelling as chk

from .check import Check


class TestCheck(Check):
    """The test class for consistency.spelling."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for consistency.spelling."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert self.passes("""The centre for the arts is the art centre.""")
        assert self.passes("""The center for the arts is the art center.""")
        assert not self.passes("""The centre of the arts is the art center.""")



"""Tests for weasel_words.very check."""

from proselint.checks.weasel_words import very as chk

from .check import Check


class TestCheck(Check):
    """The test class for weasel_words.very."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for weasel_words.very."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""The book was very interesting.""")



"""Tests for dates_times.dates check."""

from proselint.checks.dates_times import dates as chk

from .check import Check


class TestCheck(Check):
    """The test class for dates_times.dates."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke_check_decade_apostrophes_short(self):
        """Basic smoke test.

        This for the function
        dates_times.dates.check_decade_apostrophes_short.

        """
        assert chk.check_decade_apostrophes_short(
            "Basic smoke phrase without issues.") == []
        assert chk.check_decade_apostrophes_short(
            "It happened in the 90s.") == []
        assert chk.check_decade_apostrophes_short(
            "It happened in the 90's.") != []

    def test_smoke_check_decade_apostrophes_long(self):
        """Basic smoke test.

        This is for the function
        dates_times.dates.decade_apostrophes_long.

        """
        assert chk.check_decade_apostrophes_long(
            "Basic smoke phrase without issues.") == []
        assert chk.check_decade_apostrophes_long(
            "It happened in the 1980s.") == []
        assert chk.check_decade_apostrophes_long(
            "It happened in the 1980's.") != []

    def test_smoke_check_dash_and_from(self):
        """Basic smoke test.

        This for the function
        dates_times.dates.dash_and_from.

        """
        assert chk.check_dash_and_from(
            "Basic smoke phrase without issues.") == []
        assert chk.check_dash_and_from(
            "It happened from 2000 to 2005.") == []
        assert chk.check_dash_and_from(
            "It happened from 2000-2005.") != []

    def test_smoke_check_month_year_comma(self):
        """Basic smoke test.

        This is for the function
        dates_times.dates.check_month_year_comma.

        """
        assert chk.check_month_year_comma(
            "Basic smoke phrase without issues.") == []
        assert chk.check_month_year_comma(
            "It happened in August 2008.") == []
        assert chk.check_month_year_comma(
            "It happened in August, 2008.") != []

    def test_smoke_check_month_of_year(self):
        """Basic smoke test.

        This is for the function
        dates_times.dates.check_month_of_year.

        """
        assert chk.check_month_of_year(
            "Basic smoke phrase without issues.") == []
        assert chk.check_month_of_year(
            "It happened in August 2008.") == []
        assert chk.check_month_of_year(
            "It happened in August of 2008.") != []



"""Diacritical marks.

Use of diacritical marks where common.
"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "typography.diacritical_marks"
    msg = "Use diacritical marks in '{}'."

    list = [
        # French loanwords
        ["beau idéal",         ["beau ideal"]],
        ["boutonnière",        ["boutonniere"]],
        ["bric-à-brac",        ["bric-a-brac"]],
        ["café",               ["cafe"]],
        ["cause célèbre",      ["cause celebre"]],
        ["chèvre",             ["chevre"]],
        ["cliché",             ["cliche"]],
        ["comme ci comme ça",  ["comme ci comme ca", "comsi comsa"]],
        ["consommé",           ["consomme"]],
        ["coup d'état",        ["coup d'etat"]],
        ["coup de grâce",      ["coup de grace"]],
        ["crudités",           ["crudites"]],
        ["crème brûlée",       ["creme brulee"]],
        ["crème de menthe",    ["creme de menthe"]],
        ["crème fraîche",      ["creme fraice"]],
        ["crème fraîche",      ["creme fresh"]],
        ["crêpe",              ["crepe"]],
        ["débutante",          ["debutante"]],
        ["décor",              ["decor"]],
        ["déjà vu",            ["deja vu"]],
        ["dénouement",         ["denouement"]],
        ["façade",             ["facade"]],
        ["fiancé",             ["fiance"]],
        ["fiancée",            ["fiancee"]],
        ["flambé",             ["flambe"]],
        ["garçon",             ["garcon"]],
        ["lycée",              ["lycee"]],
        ["maître d",           ["maitre d"]],
        ["ménage à trois",     ["menage a trois"]],
        ["négligée",           ["negligee"]],
        ["papier-mâché", ["papier-mache", "paper mache", "paper-mache"]],
        ["protégé",            ["protege"]],
        ["protégée",           ["protegee"]],
        ["purée",              ["puree"]],
        ["raison d'être",      ["raison d'etre"]],
        ["my résumé",          ["my resume"]],
        ["your résumé",        ["your resume"]],
        ["his résumé",         ["his resume"]],
        ["her résumé",         ["her resume"]],
        ["a résumé",           ["a resume"]],
        ["the résumé",         ["the resume"]],
        ["risqué",             ["risque"]],
        ["roué",               ["roue"]],
        ["soirée",             ["soiree"]],
        ["soufflé",            ["souffle"]],
        ["soupçon",            ["soupcon"]],
        ["touché",             ["touche"]],
        ["tête-à-tête",        ["tete-a-tete"]],
        ["voilà",              ["voila"]],
        ["à la carte",         ["a la carte"]],
        ["à la mode",          ["a la mode"]],
        ["émigré",             ["emigre"]],

        # Spanish loanwords
        ["El Niño",            ["El Nino"]],
        ["jalapeño",           ["jalapeno"]],
        ["La Niña",            ["La Nina"]],
        ["piña colada",        ["pina colada"]],
        ["señor",              ["senor"]],
        ["señora",             ["senora"]],
        ["señorita",           ["senorita"]],

        # Portuguese loanwords
        ["açaí",               ["acai"]],

        # German loanwords
        ["doppelgänger",       ["doppelganger"]],
        ["Führer",             ["Fuhrer"]],
        ["Gewürztraminer",     ["Gewurztraminer"]],
        ["vis-à-vis",          ["vis-a-vis"]],
        ["Übermensch",         ["Ubermensch"]],

        # Swedish loanwords
        ["filmjölk",           ["filmjolk"]],
        ["smörgåsbord",        ["smorgasbord"]],

        # Names, places, and companies
        ["Beyoncé",            ["Beyonce"]],
        ["Brontë",             ["Bronte"]],
        ["Brontë",             ["Bronte"]],
        ["Champs-Élysées",     ["Champs-Elysees"]],
        ["Citroën",            ["Citroen"]],
        ["Curaçao",            ["Curacao"]],
        ["Häagen-Dazs",        ["Haagen-Dazs", "Haagen Dazs"]],
        ["Löwenbräu",          ["Lowenbrau"]],
        ["Monégasque",         ["Monegasque"]],
        ["Mötley Crüe",        ["Motley Crue"]],
        ["Nescafé",            ["Nescafe"]],
        ["Queensrÿche",        ["Queensryche"]],
        ["Québec",             ["Quebec"]],
        ["Québécois",          ["Quebecois"]],
        ["Ångström",           ["Angstrom"]],
        ["ångström",           ["angstrom"]],
        ["Škoda",              ["Skoda"]],
    ]

    return preferred_forms_check(text, list, err, msg)



"""Check that the CLI returns the appropriate exit code."""

from click.testing import CliRunner

from proselint.command_line import proselint

from .check import Check


class TestExitCodes(Check):
    """Test class for CLI exit codes"""

    __test__ = True

    def setUp(self):
        self.runner = CliRunner()

    def test_exit_code_demo(self):
        """Ensure that linting the demo returns an exit code of 1."""
        output = self.runner.invoke(proselint, "--demo")
        assert output.exit_code == 1

    def test_exit_code_version(self):
        """Ensure that getting the version returns an exit code of 0."""
        output = self.runner.invoke(proselint, "--version")
        assert output.exit_code == 0



"""Tests for consistency.spacing check."""

from proselint.checks.consistency import spacing as chk

from .check import Check


class TestCheck(Check):
    """The test class for consistency.spacing."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for consistency.spacing."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert self.passes(
            """This is good. Only one space each time. Every time.""")
        assert not self.passes("""This is bad.  Not consistent. At all.""")



"""Test the consistency_check function from the tools.py module."""

from proselint.tools import consistency_check as chk

from .check import Check


class TestCheck(Check):
    """The test class for tools.consistency_check."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def setUp(self):
        """Create some test fixtures."""
        self.L = [['colour', 'color']]
        self.err = 'error message'
        self.msg = 'inconsistent form of {} vs {}'

    def test_smoke(self):
        """Basic smoke test for consistency_check."""
        assert chk(
            "Painting colour on color", self.L, self.err, self.msg) != []
        assert chk(
            "Painting colour on colour", self.L, self.err, self.msg) == []
        assert chk(
            "Painting color on color", self.L, self.err, self.msg) == []



"""Test uncomparables.misc"""

from proselint.checks.uncomparables import misc as chk

from .check import Check


class TestCheck(Check):
    """The test class for uncomparables.misc."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for uncomparables.misc."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""The item was more unique.""")

    def test_sample_phrases(self):
        """Find 'very unique'."""
        assert not self.passes("""This sentence is very unique.""")

    def test_spaces(self):
        """Handle spacing correctly."""
        assert not self.passes("""This sentence is very\nunique.""")
        assert not self.passes("""Kind of complete.""")
        assert self.passes("""Every perfect instance.""")

    def test_constitutional(self):
        """Don't flag exceptions."""
        assert self.passes("""A more perfect union.""")



"""Tests for misc.phrasal_adjectives check."""

from proselint.checks.misc import phrasal_adjectives as chk

from .check import Check


class TestCheck(Check):
    """The test class for misc.phrasal_adjectives."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for misc.phrasal_adjectives."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""There were across the board discounts.""")

    def test_smoke_ly(self):
        """Basic smoke test for misc.phrasal_adjectives.check_ly."""
        assert chk.check_ly("""Smoke phrase with nothing flagged.""") == []
        assert chk.check_ly("""He ran swiftly-fast.""")
        assert chk.check_ly("""The not-so-hotly-contested
                             result was fine.""") == []



"""-ve vs. -of."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """-ve vs. -of."""
    err = "spelling.ve_of"
    msg = "-ve vs. -of. '{}' is the preferred spelling."

    preferences = [

        ["could've", ["could of"]],
        ["should've", ["should of"]],
        ["would've", ["would of"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



"""Use the right symbols.

source:     Butterick's Practical Typography
source_url: http://practicaltypography.com/
"""

from proselint.tools import existence_check, max_errors, memoize


@max_errors(3)
@memoize
def check_ellipsis(text):
    """Use an ellipsis instead of three dots."""
    err = "typography.symbols.ellipsis"
    msg = "'...' is an approximation, use the ellipsis symbol '…'."
    regex = r"\.\.\."

    return existence_check(text, [regex], err, msg, require_padding=False,
                           offset=0)


@max_errors(1)
@memoize
def check_copyright_symbol(text):
    """Use the copyright symbol instead of (c)."""
    err = "typography.symbols.copyright"
    msg = "(c) is a goofy alphabetic approximation, use the symbol ©."
    regex = r"\([cC]\)"

    return existence_check(text, [regex], err, msg, require_padding=False)


@max_errors(3)
@memoize
def check_trademark_symbol(text):
    """Use the trademark symbol instead of (TM)."""
    err = "typography.symbols.trademark"
    msg = "(TM) is a goofy alphabetic approximation, use the symbol ™."
    regex = r"\(TM\)"

    return existence_check(text, [regex], err, msg, require_padding=False)


@max_errors(3)
@memoize
def check_registered_trademark_symbol(text):
    """Use the registered trademark symbol instead of (R)."""
    err = "typography.symbols.trademark"
    msg = "(R) is a goofy alphabetic approximation, use the symbol ®."
    regex = r"\([rR]\)"

    return existence_check(text, [regex], err, msg, require_padding=False)


@max_errors(3)
@memoize
def check_sentence_spacing(text):
    """Use no more than two spaces after a period."""
    err = "typography.symbols.sentence_spacing"
    msg = "More than two spaces after the period; use 1 or 2."
    regex = r"\. {3}"

    return existence_check(text, [regex], err, msg, require_padding=False)


@max_errors(3)
@memoize
def check_multiplication_symbol(text):
    """Use the multiplication symbol ×, not the lowercase letter x."""
    err = "typography.symbols.multiplication_symbol"
    msg = "Use the multiplication symbol ×, not the letter x."
    regex = r"[0-9]+ ?x ?[0-9]+"

    return existence_check(text, [regex], err, msg, require_padding=False)


@max_errors(3)
@memoize
def check_curly_quotes(text):
    """Use curly quotes, not straight quotes."""
    err = "typography.symbols.curly_quotes"
    msg = 'Use curly quotes “”, not straight quotes "".'
    regex = r"\"[\w\s\d]+\""

    return existence_check(text, [regex], err, msg, require_padding=False)

# @memoize
# def check_en_dash_separated_names(text):
#     """Use an en-dash to separate names."""
#     # [u"[A-Z][a-z]{1,10}[-\u2014][A-Z][a-z]{1,10}",
#     #     u"Use an en dash (–) to separate names."],



"""Too much yelling.

---
layout:     post
source:     ???
source_url: ???
title:      yelling
date:       2014-06-10 12:31:19
categories: writing
---

Too much yelling.

"""
from proselint.tools import existence_check, max_errors, memoize, ppm_threshold


@max_errors(1)
@memoize
def check_repeated_exclamations(text):
    """Check the text."""
    err = "leonard.exclamation.multiple"
    msg = "Stop yelling. Keep your exclamation points under control."

    regex = r"[\!]\s*?[\!]{1,}"

    return existence_check(text, [regex], err, msg, require_padding=False,
                           ignore_case=False, dotall=True)


@ppm_threshold(30)
@memoize
def check_exclamations_ppm(text):
    """Make sure that the exclamation ppm is under 30."""
    err = "leonard.exclamation.30ppm"
    msg = "More than 30 ppm of exclamations. Keep them under control."

    regex = r"\w!"

    return existence_check(text, [regex], err, msg, require_padding=False)



"""Tests for misc.narcissism check."""

from proselint.checks.misc import narcissism as chk

from .check import Check


class TestCheck(Check):
    """The test class for misc.narcissism."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for misc.narcissism."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes(
            """In recent years, an increasing number of scientists have studied
             the problem in detail.""")



"""Tests for misc.illogic check."""

from proselint.checks.misc import illogic as chk

from .check import Check


class TestCheck(Check):
    """The test class for misc.illogic."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for misc.illogic."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert not self.passes("""We should preplan the trip.""")

    def test_smoke_coin_a_phrase_from(self):
        """Basic smoke test for misc.illogic.check_coin_a_phrase_from."""
        assert chk.check_coin_a_phrase_from(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_coin_a_phrase_from(
            """To coin a phrase from him, No diggity""") != []

    def test_smoke_check_without_your_collusion(self):
        """Basic smoke test for misc.illogic."""
        assert chk.check_without_your_collusion(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_without_your_collusion(
            """Not Without your collusion you won't'.""") != []



"""Tests for typography.exclamation check."""

from proselint.checks.typography import exclamation as chk

from .check import Check


class TestCheck(Check):
    """The test class for typography.exclamation."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke_repeated_exclamations(self):
        """Basic smoke test.

        Test for typography.exclamation.check_repeated_exclamations.
        """
        assert chk.check_repeated_exclamations(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_repeated_exclamations(
            """I'm really excited!!""") != []

    def test_smoke_exclamations_ppm(self):
        """Basic smoke test.

        Test for typography.exclamation.check_exclamations_ppm.
        """
        assert chk.check_exclamations_ppm(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_exclamations_ppm(
            """I'm really excited! Really!""") != []



"""Tests for lgbtq.terms check."""

from proselint.checks.lgbtq import offensive_terms as chk

from .check import Check


class TestCheck(Check):
    """The test class for lgbtq.offensive_terms."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for lgbtq.offensive_terms."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert self.passes("""I once met a gay man.""")
        assert not self.passes("""I once met a fag.""")



"""General-purpose tools shared across linting checks."""

import copy
import dbm
import functools
import hashlib
import importlib
import inspect
import json
import os
import re
import shelve
import sys
import traceback
from warnings import showwarning as warn

from . import config

_cache_shelves = dict()
proselint_path = os.path.dirname(os.path.realpath(__file__))
home_dir = os.path.expanduser("~")
cwd = os.getcwd()


def close_cache_shelves():
    """Close previously opened cache shelves."""
    for cache in _cache_shelves.values():
        cache.close()
    _cache_shelves.clear()


def close_cache_shelves_after(f):
    """Decorate a function to ensure cache shelves are closed after call."""
    @functools.wraps(f)
    def wrapped(*args, **kwargs):
        f(*args, **kwargs)
        close_cache_shelves()
    return wrapped


def _get_xdg_path(variable_name, default_path):
    path = os.environ.get(variable_name)
    if path is None or path == '':
        return default_path
    else:
        return path


def _get_xdg_config_home():
    return _get_xdg_path('XDG_CONFIG_HOME', os.path.join(home_dir, '.config'))


def _get_xdg_cache_home():
    return _get_xdg_path('XDG_CACHE_HOME', os.path.join(home_dir, '.cache'))


def _get_cache(cachepath):
    if cachepath in _cache_shelves:
        return _cache_shelves[cachepath]

    try:
        cache = shelve.open(cachepath, protocol=2)
    except dbm.error:
        # dbm error on open - delete and retry
        print('Error (%s) opening %s - will attempt to delete and re-open.' %
              (sys.exc_info()[1], cachepath))
        try:
            os.remove(cachepath)
            cache = shelve.open(cachepath, protocol=2)
        except Exception:
            print('Error on re-open: %s' % sys.exc_info()[1])
            cache = None
    except Exception:
        # unknown error
        print('Could not open cache file %s, maybe name collision. '
              'Error: %s' % (cachepath, traceback.format_exc()))
        cache = None

    # Don't fail on bad caches
    if cache is None:
        print('Using in-memory shelf for cache file %s' % cachepath)
        cache = shelve.Shelf(dict())

    _cache_shelves[cachepath] = cache
    return cache


def memoize(f):
    """Cache results of computations on disk."""
    # Determine the location of the cache.
    cache_dirname = os.path.join(_get_xdg_cache_home(), 'proselint')
    legacy_cache_dirname = os.path.join(home_dir, ".proselint")

    if not os.path.isdir(cache_dirname):
        # Migrate the cache from the legacy path to XDG compliant location.
        if os.path.isdir(legacy_cache_dirname):
            os.rename(legacy_cache_dirname, cache_dirname)
        # Create the cache if it does not already exist.
        else:
            os.makedirs(cache_dirname)

    cache_filename = f.__module__ + "." + f.__name__
    cachepath = os.path.join(cache_dirname, cache_filename)

    @functools.wraps(f)
    def wrapped(*args, **kwargs):

        # handle instance methods
        if hasattr(f, '__self__'):
            args = args[1:]

        signature = cache_filename.encode("utf-8")

        tempargdict = inspect.getcallargs(f, *args, **kwargs)

        for item in list(tempargdict.items()):
            if item[0] == "text":
                signature += item[1].encode("utf-8")

        key = hashlib.sha256(signature).hexdigest()
        cache = _get_cache(cachepath)

        try:
            return cache[key]
        except KeyError:
            value = f(*args, **kwargs)
            cache[key] = value
            cache.sync()
            return value
        except TypeError:
            call_to = f.__module__ + '.' + f.__name__
            print('Warning: could not disk cache call to %s;'
                  'it probably has unhashable args. Error: %s' %
                  (call_to, traceback.format_exc()))
            return f(*args, **kwargs)

    return wrapped


def get_checks(options):
    """Extract the checks."""
    sys.path.append(proselint_path)
    checks = []
    check_names = [key for (key, val) in options["checks"].items() if val]

    for check_name in check_names:
        module = importlib.import_module("checks." + check_name)
        for d in dir(module):
            if re.match("check", d):
                checks.append(getattr(module, d))

    return checks


def deepmerge_dicts(dict1, dict2):
    """Deep merge dictionaries, second dict will take priority."""
    result = copy.deepcopy(dict1)

    for key, value in dict2.items():
        if isinstance(value, dict):
            result[key] = deepmerge_dicts(result[key] or {}, value)
        else:
            result[key] = value

    return result


def load_options(config_file_path=None, conf_default=None):
    """Read various proselintrc files, allowing user overrides."""
    conf_default = conf_default or {}
    if os.path.isfile("/etc/proselintrc"):
        conf_default = json.load(open("/etc/proselintrc"))

    user_config_paths = [
        os.path.join(cwd, '.proselintrc.json'),
        os.path.join(_get_xdg_config_home(), 'proselint', 'config.json'),
        os.path.join(home_dir, '.proselintrc.json')
    ]

    if config_file_path:
        if not os.path.isfile(config_file_path):
            raise FileNotFoundError(
                f"Config file {config_file_path} does not exist")
        user_config_paths.insert(0, config_file_path)

    user_options = {}
    for path in user_config_paths:
        if os.path.isfile(path):
            user_options = json.load(open(path))
            break
        oldpath = path.replace(".json", "")
        if os.path.isfile(oldpath):
            warn(f"{oldpath} was found instead of a JSON file."
                 f" Rename to {path}.", DeprecationWarning, "", 0)
            user_options = json.load(open(oldpath))
            break

    return deepmerge_dicts(conf_default, user_options)


def errors_to_json(errors):
    """Convert the errors to JSON."""
    out = []
    for e in errors:
        out.append({
            "check": e[0],
            "message": e[1],
            "line": 1 + e[2],
            "column": 1 + e[3],
            "start": 1 + e[4],
            "end": 1 + e[5],
            "extent": e[6],
            "severity": e[7],
            "replacements": e[8],
        })

    return json.dumps(
        {"status": "success", "data": {"errors": out}}, sort_keys=True)


def line_and_column(text, position):
    """Return the line number and column of a position in a string."""
    position_counter = 0
    line_no = 0
    for line in text.splitlines(True):
        if (position_counter + len(line.rstrip())) >= position:
            break
        position_counter += len(line)
        line_no += 1
    return (line_no, position - position_counter)


def lint(input_file, debug=False, config=config.default):
    """Run the linter on the input file."""
    if isinstance(input_file, str):
        text = input_file
    else:
        text = input_file.read()

    # Get the checks.
    checks = get_checks(config)

    # Apply all the checks.
    errors = []
    for check in checks:

        result = check(text)

        for error in result:
            (start, end, check, message, replacements) = error
            (line, column) = line_and_column(text, start)
            if not is_quoted(start, text):
                errors += [(check, message, line, column, start, end,
                            end - start, "warning", replacements)]

        if len(errors) > config["max_errors"]:
            break

    # Sort the errors by line and column number.
    errors = sorted(errors[:config["max_errors"]], key=lambda e: (e[2], e[3]))

    return errors


def assert_error(text, check, n=1):
    """Assert that text has n errors of type check."""
    assert_error.description = f"No {check} error for '{text}'"
    assert len([error[0] for error in lint(text) if error[0] == check]) == n


def consistency_check(text, word_pairs, err, msg, offset=0):
    """Build a consistency checker for the given word_pairs."""
    errors = []

    msg = " ".join(msg.split())

    for w in word_pairs:
        matches = [
            [m for m in re.finditer(w[0], text)],
            [m for m in re.finditer(w[1], text)]
        ]

        if len(matches[0]) > 0 and len(matches[1]) > 0:

            idx_minority = len(matches[0]) > len(matches[1])

            for m in matches[idx_minority]:
                errors.append((
                    m.start() + offset,
                    m.end() + offset,
                    err,
                    msg.format(w[~idx_minority], m.group(0)),
                    w[~idx_minority]))

    return errors


def preferred_forms_check(text, list, err, msg, ignore_case=True, offset=0):
    """Build a checker that suggests the preferred form."""
    if ignore_case:
        flags = re.IGNORECASE
    else:
        flags = 0

    msg = " ".join(msg.split())

    errors = []
    regex = r"[\W^]{}[\W$]"
    for p in list:
        for r in p[1]:
            for m in re.finditer(regex.format(r), text, flags=flags):
                txt = m.group(0).strip()
                errors.append((
                    m.start() + 1 + offset,
                    m.end() + offset,
                    err,
                    msg.format(p[0], txt),
                    p[0]))

    return errors


def existence_check(text, list, err, msg, ignore_case=True, str=False,
                    offset=0, require_padding=True, dotall=False,
                    excluded_topics=None, exceptions=(), join=False):
    """Build a checker that prohibits certain words or phrases."""
    flags = 0

    msg = " ".join(msg.split())

    if ignore_case:
        flags = flags | re.IGNORECASE

    if str:
        flags = flags | re.UNICODE

    if dotall:
        flags = flags | re.DOTALL

    if require_padding:
        regex = r"(?:^|\W){}[\W$]"
    else:
        regex = r"{}"

    errors = []

    # If the topic of the text is in the excluded list, return immediately.
    if excluded_topics:
        tps = topics(text)
        if any([t in excluded_topics for t in tps]):
            return errors

    rx = "|".join(regex.format(w) for w in list)
    for m in re.finditer(rx, text, flags=flags):
        txt = m.group(0).strip()
        if any([re.search(exception, txt) for exception in exceptions]):
            continue
        errors.append((
            m.start() + 1 + offset,
            m.end() + offset,
            err,
            msg.format(txt),
            None))

    return errors


def max_errors(limit):
    """Decorate a check to truncate error output to a specified limit."""
    def wrapper(f):
        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            return truncate_errors(f(*args, **kwargs), limit)
        return wrapped
    return wrapper


def truncate_errors(errors, limit=float("inf")):
    """If limit was specified, truncate the list of errors.

    Give the total number of times that the error was found elsewhere.
    """
    if len(errors) > limit:
        start1, end1, err1, msg1, replacements = errors[0]

        if len(errors) == limit + 1:
            msg1 += " Found once elsewhere."
        else:
            msg1 += f" Found {len(errors)} times elsewhere."

        errors = [(start1, end1, err1, msg1, replacements)] + errors[1:limit]

    return errors


def ppm_threshold(threshold):
    """Decorate a check to error if the PPM threshold is surpassed."""
    def wrapped(f):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            return threshold_check(f(*args, **kwargs), threshold, len(args[0]))
        return wrapper
    return wrapped


def threshold_check(errors, threshold, length):
    """Check that returns an error if the PPM threshold is surpassed."""
    if length > 0:
        errcount = len(errors)
        ppm = (errcount / length) * 1e6

        if ppm >= threshold and errcount >= 1:
            return [errors[0]]
    return []


def is_quoted(position, text):
    """Determine if the position in the text falls within a quote."""
    def matching(quotemark1, quotemark2):
        straight = '\"\''
        curly = '“”'
        if quotemark1 in straight and quotemark2 in straight:
            return True
        if quotemark1 in curly and quotemark2 in curly:
            return True
        else:
            return False

    def find_ranges(text):
        s = 0
        q = pc = ''
        start = None
        ranges = []
        seps = " .,:;-\r\n"
        quotes = ['\"', '“', '”', "'"]
        for i, c in enumerate(text + "\n"):
            if s == 0 and c in quotes and pc in seps:
                start = i
                s = 1
                q = c
            elif s == 1 and matching(c, q):
                s = 2
            elif s == 2:
                if c in seps:
                    ranges.append((start+1, i-1))
                    start = None
                    s = 0
                else:
                    s = 1
            pc = c
        return ranges

    def position_in_ranges(ranges, position):
        for start, end in ranges:
            if start <= position < end:
                return True
        return False

    return position_in_ranges(find_ranges(text), position)


def detector_50_Cent(text):
    """Determine whether 50 Cent is a topic."""
    keywords = [
        "50 Cent",
        "rap",
        "hip hop",
        "Curtis James Jackson III",
        "Curtis Jackson",
        "Eminem",
        "Dre",
        "Get Rich or Die Tryin'",
        "G-Unit",
        "Street King Immortal",
        "In da Club",
        "Interscope",
    ]
    num_keywords = sum(word in text for word in keywords)
    return ("50 Cent", float(num_keywords > 2))


def topics(text):
    """Return a list of topics."""
    detectors = [
        detector_50_Cent
    ]
    ts = []
    for detector in detectors:
        ts.append(detector(text))

    return [t[0] for t in ts if t[1] > 0.95]


def context(text, position, level="paragraph"):
    """Get sentence or paragraph that surrounds the given position."""
    if level == "sentence":
        pass
    elif level == "paragraph":
        pass

    return ""



"""Tests for mixed_metaphors.misc check."""

from proselint.checks.mixed_metaphors import misc as chk

from .check import Check


class TestCheck(Check):
    """The test class for mixed_metaphors.misc."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke_bottleneck(self):
        """Basic smoke test for check_bottleneck."""
        assert chk.check_bottleneck(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_bottleneck(
            """The project produced a huge bottleneck.""") != []

    def test_smoke_misc(self):
        """Basic smoke test for check_misc."""
        assert chk.check_misc(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_misc(
            """Writing tests is not rocket surgery.""") != []



"""Check that the CLI can handle invalid characters."""

from pathlib import Path

from click.testing import CliRunner

from proselint.command_line import proselint

from .check import Check

CHAR_FILE = Path(__file__, "../invalid-chars.txt").resolve()


class TestInvalidCharacters(Check):
    """Test class for testing invalid characters on the CLI"""

    __test__ = True

    def test_invalid_characters(self):
        """Ensure that invalid characters do not break proselint."""
        runner = CliRunner()

        output = runner.invoke(proselint, CHAR_FILE)

        assert "UnicodeDecodeError" not in output.stdout
        assert "FileNotFoundError" not in output.stdout



"""Test garner.dates."""

from proselint.checks.typography import exclamation

from .check import Check


class TestCheck(Check):
    """Test class for leonard.exclamation."""

    __test__ = True

    def test_capitalization_and_no_exclamation(self):
        """Don't throw error when phrase has capitalization."""
        text = """
             The QUICK BROWN fox juMPED over the lazy cat.
        """
        errors = exclamation.check_repeated_exclamations(text)
        assert len(errors) == 0

    def test_exclamation(self):
        """Test leonard.exclamation. with exclamation marks."""
        text = """Sally sells seashells and they were too expensive!!!!"""
        errors = exclamation.check_repeated_exclamations(text)
        print(errors)
        assert len(errors) == 1



"""-ally vs. -ly."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """-ally vs. -ly."""
    err = "spelling.ally_ly"
    msg = "-ally vs. -ly. '{}' is the correct spelling."

    preferences = [
        ["academically", ["academicly"]],
        ["accidentally", ["accidently"]],
        ["automatically", ["automaticly"]],
        ["basically", ["basicly"]],
        ["dramatically", ["dramaticly"]],
        ["emotionally", ["emotionly"]],
        ["incidentally", ["incidently"]],
        ["optimistically", ["optimisticly"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



"""Test the existence_check function from the tools.py module."""
from proselint.tools import existence_check as chk

from .check import Check


class TestCheck(Check):
    """The test class for tools.existence_check."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def setUp(self):
        """setUp method creating some test fixtures."""
        self.L = ['abc']
        self.err = 'error message'
        self.msg = 'it exists'

    def test_smoke(self):
        """Basic smoke test to determine that existence_check is working."""
        assert chk(
            """abc is as easy as 123""", self.L, self.err, self.msg) != []
        assert chk(
            """easy breezy""", self.L, self.err, self.msg) == []
        assert self.err in chk(
            """abc is as easy as 123""", self.L, self.err, self.msg)[0]
        assert self.msg in chk(
            """abc is as easy as 123""", self.L, self.err, self.msg)[0]

    def test_multiple_matches(self):
        """Test that multiple matches are found correctly."""
        assert len(
            chk("""abc and abc are as easy as 123""",
                self.L, self.err, self.msg)) == 2
        assert len(
            chk("""ABC and abc are as easy as 123""",
                self.L, self.err, self.msg, ignore_case=True)) == 2
        assert len(
            chk("""ABC and abc are as easy as 123""",
                self.L, self.err, self.msg, ignore_case=False)) == 1
        assert chk(
            """abcabc are easy as 123""", self.L, self.err, self.msg) == []

    def test_string_types(self):
        """Test that the function works with different string types."""
        assert chk('abc is easy as 123', self.L, self.err, self.msg) != []
        assert chk("abc is easy as 123", self.L, self.err, self.msg) != []
        assert chk(u'abc is easy as 123', self.L, self.err, self.msg) != []
        assert chk(u"abc is easy as 123", self.L, self.err, self.msg) != []

    def test_exceptions(self):
        """Test that existence_check does not report excluded phrases"""
        regex = [r"\b(\w+)\b\s\1"]
        no = ["should should"]
        errs = chk("should should flag flag.", regex, "", "",
                   require_padding=False)
        assert len(errs) == 2
        errs = chk("should should flag flag.", regex, "", "", exceptions=no,
                   require_padding=False)
        assert len(errs) == 1



"""Proselint config - replacement for default .proselintrc since #1212."""

default = {
    "max_errors": 1000,
    "checks": {
        "airlinese.misc": True,
        "annotations.misc": True,
        "archaism.misc": True,
        "cliches.hell": True,
        "cliches.misc": True,
        "consistency.spacing": True,
        "consistency.spelling": True,
        "corporate_speak.misc": True,
        "cursing.filth": True,
        "cursing.nfl": False,
        "cursing.nword": True,
        "dates_times.am_pm": True,
        "dates_times.dates": True,
        "hedging.misc": True,
        "hyperbole.misc": True,
        "jargon.misc": True,
        "lexical_illusions.misc": True,
        "lgbtq.offensive_terms": True,
        "lgbtq.terms": True,
        "links.broken": False,
        "malapropisms.misc": True,
        "misc.apologizing": True,
        "misc.back_formations": True,
        "misc.bureaucratese": True,
        "misc.but": True,
        "misc.capitalization": True,
        "misc.chatspeak": True,
        "misc.commercialese": True,
        "misc.composition": True,
        "misc.currency": True,
        "misc.debased": True,
        "misc.false_plurals": True,
        "misc.illogic": True,
        "misc.inferior_superior": True,
        "misc.institution_name": True,
        "misc.latin": True,
        "misc.many_a": True,
        "misc.metaconcepts": True,
        "misc.metadiscourse": True,
        "misc.narcissism": True,
        "misc.not_guilty": True,
        "misc.phrasal_adjectives": True,
        "misc.preferred_forms": True,
        "misc.pretension": True,
        "misc.professions": True,
        "misc.punctuation": True,
        "misc.scare_quotes": True,
        "misc.suddenly": True,
        "misc.tense_present": True,
        "misc.waxed": True,
        "misc.whence": True,
        "mixed_metaphors.misc": True,
        "mondegreens.misc": True,
        "needless_variants.misc": True,
        "nonwords.misc": True,
        "oxymorons.misc": True,
        "psychology.misc": True,
        "redundancy.misc": True,
        "redundancy.ras_syndrome": True,
        "skunked_terms.misc": True,
        "spelling.able_atable": True,
        "spelling.able_ible": True,
        "spelling.ally_ly": True,
        "spelling.ance_ence": True,
        "spelling.athletes": True,
        "spelling.ely_ly": True,
        "spelling.em_im_en_in": True,
        "spelling.er_or": True,
        "spelling.in_un": True,
        "spelling.misc": True,
        "spelling.ve_of": True,
        "security.credit_card": True,
        "security.password": True,
        "sexism.misc": True,
        "terms.animal_adjectives": True,
        "terms.denizen_labels": True,
        "terms.eponymous_adjectives": True,
        "terms.venery": True,
        "typography.diacritical_marks": True,
        "typography.exclamation": True,
        "typography.symbols": True,
        "uncomparables.misc": True,
        "weasel_words.misc": True,
        "weasel_words.very": True
    }
}



"""-ely vs. -ly."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """-ely vs. -ly."""
    err = "spelling.ely_ly"
    msg = "-ely vs. -ly. '{}' is the correct spelling."

    preferences = [

        ["completely", ["completly"]],
        ["immediately", ["immediatly"]],
        ["unfortunately", ["unfortunatly"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



#!/usr/bin/env python

"""Test GMEU entry 'a', part B."""


from proselint.tools import assert_error

from tests.check import Check


class chk:
    def check(self, text):
        return assert_error, text, "misc.a_distributive"


class TestCheck(Check):
    """The test class for GMEU entry A - using a over per in the distributive
    sense.
    """

    @property
    def this_check(self):
        return chk

    def test_smoke(self):
        sentences = [
            "An apple per day keeps the doctor away.",
            "I sleep eight hours per night.",
            "Their policy allows one golf cart a couple.",
            "The company donated five books a student.",
            "Our a-unit cost is less than $1000.",
            "The $50-a-parent fee seems unreasonably high."
        ]
        for sentence in sentences:
            assert not self.passes(sentence)



"""Tests for lgbtq.offensive_terms check."""

from proselint.checks.lgbtq import terms as chk

from .check import Check


class TestCheck(Check):
    """The test class for lgbtq.terms."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for lgbtq.terms."""
        assert self.passes("""Smoke phrase with nothing flagged.""")
        assert self.passes("""They were a gay couple.""")
        assert not self.passes("""He was a homosexual man.""")

    def test_homosexual_term(self):
        """Check that the term homosexual does not get caught."""
        assert self.passes("""Homosexual.""")

    def test_sexual_prefence(self):
        """Check that sexual preference is flagged."""
        assert not self.passes("""My sexual preference is for women.""")



"""Tests for lexical_illusions.misc check."""

from proselint.checks.lexical_illusions import misc as chk

from .check import Check


class TestCheck(Check):
    """The test class for lexical_illusions.misc."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke(self):
        """Basic smoke test for lexical_illusions.misc."""
        assert self.passes("Smoke phrase with nothing flagged.")
        assert not self.passes("Paris in the the springtime.")
        assert self.passes("And he's gone, gone with the breeze")
        assert self.passes("You should know that that sentence wasn't wrong.")
        assert self.passes("She had had dessert on the balcony.")
        assert not self.passes("You should know that that that was wrong.")
        assert self.passes("The practitioner's side")
        assert self.passes("An antimatter particle")
        assert self.passes("The theory")
        assert self.passes("She had coffee at the Foo-bar bar.")



"""Tests for psychology.misc check."""

from proselint.checks.psychology import misc as chk

from .check import Check


class TestCheck(Check):
    """The test class for psychology.misc."""

    __test__ = True

    @property
    def this_check(self):
        """Boilerplate."""
        return chk

    def test_smoke_lie_detector_test(self):
        """Basic smoke test for psychology.misc.check_lie_detector_test."""
        assert chk.check_lie_detector_test(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_lie_detector_test(
            """The defendant took a lie detector test.""") != []

    def test_smoke_p_equals_zero(self):
        """Basic smoke test for psychology.misc.check_p_equals_zero."""
        assert chk.check_p_equals_zero(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_p_equals_zero(
            """The effect was highly signficant at p = 0.00.""") != []

    def test_smoke_mental_telepathy(self):
        """Basic smoke test for psychology.misc.check_mental_telepathy."""
        assert chk.check_mental_telepathy(
            """Smoke phrase with nothing flagged.""") == []
        assert chk.check_mental_telepathy(
            """I've been practicing mental telepathy.""") != []
